#!/usr/bin/env python
# encoding: utf-8
"""
protocols.py - handle data confoming the the Hudora standard protocols.
See http://github.com/hudora/huTools/tree/master/doc/standards for details.

Created by Maximillian Dornseif on 2009-12-27.
Copyright (c) 2009, 2011 HUDORA. All rights reserved.

Code is based on http://code.activestate.com/recipes/573463/
"""

import warnings
warnings.warn("huTools.protocols is deprecated use cs.messaging", DeprecationWarning, stacklevel=2)

from huTools.structured import make_struct

if __name__ == '__main__':
    print "NICHT MEHR NUTZEN!"
    make_struct()
